@@ -25,8 +25,8 @@ from contextlib import closing
 
 from nose.plugins.attrib import attr
 
-from botocore.vendored.requests import adapters
-from botocore.vendored.requests.exceptions import ConnectionError
+from botocore.endpoint import Endpoint
+from botocore.exceptions import ConnectionClosedError
 from botocore.compat import six, zip_longest
 import botocore.session
 import botocore.auth
@@ -315,6 +315,11 @@ class TestS3Objects(TestS3BaseWithBucket):
         body = '*' * (5 * (1024 ** 2))
         self.assert_can_put_object(body)
 
+    def test_can_put_object_bytearray(self):
+        body_bytes = b'*' * 1024
+        body = bytearray(body_bytes)
+        self.assert_can_put_object(body)
+
     def test_get_object_stream_wrapper(self):
         self.create_object('foobarbaz', body='body contents')
         response = self.client.get_object(
@@ -813,18 +818,18 @@ class TestS3SigV4Client(BaseS3ClientTest):
     def test_request_retried_for_sigv4(self):
         body = six.BytesIO(b"Hello world!")
 
-        original_send = adapters.HTTPAdapter.send
+        original_send = Endpoint._send
         state = mock.Mock()
         state.error_raised = False
 
-        def mock_http_adapter_send(self, *args, **kwargs):
+        def mock_endpoint_send(self, *args, **kwargs):
             if not state.error_raised:
                 state.error_raised = True
-                raise ConnectionError("Simulated ConnectionError raised.")
+                raise ConnectionClosedError(endpoint_url='')
             else:
                 return original_send(self, *args, **kwargs)
-        with mock.patch('botocore.vendored.requests.adapters.HTTPAdapter.send',
-                        mock_http_adapter_send):
+        # TODO: fix with stubber / before send event
+        with mock.patch('botocore.endpoint.Endpoint._send', mock_endpoint_send):
             response = self.client.put_object(Bucket=self.bucket_name,
                                               Key='foo.txt', Body=body)
             self.assert_status_code(response, 200)
