@@ -19,8 +19,8 @@ from nose.tools import assert_equal, assert_true
 from botocore import xform_name
 import botocore.session
 from botocore.client import ClientError
-from botocore.vendored.requests import adapters
-from botocore.vendored.requests.exceptions import ConnectionError
+from botocore.endpoint import Endpoint
+from botocore.exceptions import ConnectionClosedError
 
 
 # Mapping of service -> api calls to try.
@@ -40,7 +40,7 @@ SMOKE_TESTS = {
                     'ListStacks': {}},
  'cloudfront': {'ListDistributions': {},
                 'ListStreamingDistributions': {}},
- 'cloudhsm': {'ListAvailableZones': {}},
+ 'cloudhsmv2': {'DescribeBackups': {}},
  'cloudsearch': {'DescribeDomains': {},
                  'ListDomainNames': {}},
  'cloudtrail': {'DescribeTrails': {}},
@@ -91,6 +91,7 @@ SMOKE_TESTS = {
  's3': {'ListBuckets': {}},
  'sdb': {'ListDomains': {}},
  'ses': {'ListIdentities': {}},
+ 'shield': {'GetSubscriptionState': {}},
  'sns': {'ListTopics': {}},
  'sqs': {'ListQueues': {}},
  'ssm': {'ListDocuments': {}},
@@ -127,7 +128,7 @@ ERROR_TESTS = {
         'TemplateURL': 'http://s3.amazonaws.com/foo/bar',
         }},
     'cloudfront': {'GetDistribution': {'Id': 'fake-id'}},
-    'cloudhsm': {'DescribeHapg': {'HapgArn': 'bogus-arn'}},
+    'cloudhsmv2': {'ListTags': {'ResourceId': 'fake-id'}},
     'cloudsearch': {'DescribeIndexFields': {'DomainName': 'fakedomain'}},
     'cloudtrail': {'DeleteTrail': {'Name': 'fake-trail'}},
     'cloudwatch': {'SetAlarmState': {
@@ -142,7 +143,7 @@ ERROR_TESTS = {
     'cognito-identity': {'DescribeIdentityPool': {'IdentityPoolId': 'fake'}},
     'cognito-sync': {'DescribeIdentityPoolUsage': {'IdentityPoolId': 'fake'}},
     'config': {
-        'GetResourceConfigHistory': {'resourceType': '', 'resourceId': ''},
+        'GetResourceConfigHistory': {'resourceType': '', 'resourceId': 'fake'},
         },
     'datapipeline': {'GetPipelineDefinition': {'pipelineId': 'fake'}},
     'devicefarm': {'GetDevice': {'arn': 'arn:aws:devicefarm:REGION::device:f'}},
@@ -285,15 +286,15 @@ def test_client_can_retry_request_properly():
 
 def _make_client_call_with_errors(client, operation_name, kwargs):
     operation = getattr(client, xform_name(operation_name))
-    original_send = adapters.HTTPAdapter.send
-    def mock_http_adapter_send(self, *args, **kwargs):
+    original_send = Endpoint._send
+    def mock_endpoint_send(self, *args, **kwargs):
         if not getattr(self, '_integ_test_error_raised', False):
             self._integ_test_error_raised = True
-            raise ConnectionError("Simulated ConnectionError raised.")
+            raise ConnectionClosedError(endpoint_url='')
         else:
             return original_send(self, *args, **kwargs)
-    with mock.patch('botocore.vendored.requests.adapters.HTTPAdapter.send',
-                    mock_http_adapter_send):
+    # TODO: fix with stubber / before send event
+    with mock.patch('botocore.endpoint.Endpoint._send', mock_endpoint_send):
         try:
             response = operation(**kwargs)
         except ClientError as e:
