@@ -327,13 +327,15 @@ class TestSessionPartitionFiles(BaseSessionTest):
     def test_lists_partitions_on_disk(self):
         mock_resolver = mock.Mock()
         mock_resolver.get_available_partitions.return_value = ['foo']
-        self.session.register_component('endpoint_resolver', mock_resolver)
-        self.assertEquals(['foo'], self.session.get_available_partitions())
+        self.session._register_internal_component(
+            'endpoint_resolver', mock_resolver)
+        self.assertEqual(['foo'], self.session.get_available_partitions())
 
     def test_proxies_list_endpoints_to_resolver(self):
         resolver = mock.Mock()
         resolver.get_available_endpoints.return_value = ['a', 'b']
-        self.session.register_component('endpoint_resolver', resolver)
+        self.session._register_internal_component(
+            'endpoint_resolver', resolver)
         self.session.get_available_regions('foo', 'bar', True)
 
     def test_provides_empty_list_for_unknown_service_regions(self):
@@ -355,6 +357,20 @@ class TestSessionUserAgent(BaseSessionTest):
         self.assertTrue(
             self.session.user_agent().endswith('custom-thing/other'))
 
+    def test_execution_env_not_set(self):
+        self.assertFalse(self.session.user_agent().endswith('FooEnv'))
+
+    def test_execution_env_set(self):
+        self.environ['AWS_EXECUTION_ENV'] = 'FooEnv'
+        self.assertTrue(self.session.user_agent().endswith(' exec-env/FooEnv'))
+
+    def test_agent_extra_and_exec_env(self):
+        self.session.user_agent_extra = 'custom-thing/other'
+        self.environ['AWS_EXECUTION_ENV'] = 'FooEnv'
+        user_agent = self.session.user_agent()
+        self.assertTrue(user_agent.endswith('custom-thing/other'))
+        self.assertIn('exec-env/FooEnv', user_agent)
+
 
 class TestConfigLoaderObject(BaseSessionTest):
     def test_config_loader_delegation(self):
@@ -432,6 +448,20 @@ class TestCreateClient(BaseSessionTest):
                          "explicit credentials were provided to the "
                          "create_client call.")
 
+    def test_cred_provider_called_when_partial_creds_provided(self):
+        with self.assertRaises(botocore.exceptions.PartialCredentialsError):
+            self.session.create_client(
+                'sts', 'us-west-2',
+                aws_access_key_id='foo',
+                aws_secret_access_key=None
+            )
+        with self.assertRaises(botocore.exceptions.PartialCredentialsError):
+            self.session.create_client(
+                'sts', 'us-west-2',
+                aws_access_key_id=None,
+                aws_secret_access_key='foo',
+            )
+
     @mock.patch('botocore.client.ClientCreator')
     def test_config_passed_to_client_creator(self, client_creator):
         # Make sure there is no default set
@@ -625,6 +655,30 @@ class TestCreateClient(BaseSessionTest):
             self.assertEqual(call_kwargs['api_version'], override_api_version)
 
 
+class TestSessionComponent(BaseSessionTest):
+    def test_internal_component(self):
+        component = object()
+        self.session._register_internal_component('internal', component)
+        self.assertIs(
+            self.session._get_internal_component('internal'), component)
+        with self.assertRaises(ValueError):
+            self.session.get_component('internal')
+
+    def test_internal_endpoint_resolver_is_same_as_deprecated_public(self):
+        endpoint_resolver = self.session._get_internal_component(
+            'endpoint_resolver')
+        self.assertIs(
+            self.session.get_component('endpoint_resolver'), endpoint_resolver)
+
+    def test_internal_exceptions_factory_is_same_as_deprecated_public(self):
+        exceptions_factory = self.session._get_internal_component(
+            'exceptions_factory')
+        self.assertIs(
+            self.session.get_component('exceptions_factory'),
+            exceptions_factory
+        )
+
+
 class TestComponentLocator(unittest.TestCase):
     def setUp(self):
         self.components = botocore.session.ComponentLocator()
